using System;
using System.ComponentModel;
using System.Drawing;

namespace NinjaTrader.Gui.Chart
{
    public sealed class SmartRenko : ChartStyle
    {
#pragma warning disable 169
        private static bool registered = Register(new SmartRenko());
#pragma warning restore 169

        private readonly SolidBrush _downBrush = new SolidBrush(Color.Red);
        private readonly SolidBrush _upBrush = new SolidBrush(Color.LightGreen);

        public SmartRenko()
            : base(ChartStyleType.Custom6)
        {
            BarWidth = 3;
            DownColor = Color.Red;
            UpColor = Color.LightGreen;
        }

        public override object Clone()
        {
            SmartRenko ret = new SmartRenko();
            ret.BarWidth = BarWidth;
            ret.DownColor = DownColor;
            ret.Pen = Gui.Globals.Clone(Pen);
            ret.Pen2 = Gui.Globals.Clone(Pen2);
            ret.UpColor = UpColor;
            return ret;
        }

        public override string DisplayName
        {
            get { return "SmartRenko"; }
        }

        public override void Dispose()
        {
            base.Dispose();

            _downBrush.Dispose();
            _upBrush.Dispose();
        }

        public override int GetBarPaintWidth(int barWidth)
        {
            return (int)(1 + 2 * (barWidth - 1) + 2 * Pen.Width);
        }

        public override PropertyDescriptorCollection GetProperties(PropertyDescriptor propertyDescriptor, ChartStyle chartStyle, Attribute[] attributes)
        {
            PropertyDescriptorCollection properties = base.GetProperties(propertyDescriptor, chartStyle, attributes);

            Design.DisplayNameAttribute.SetDisplayName(properties, "BarWidthUI", "\r\r\rBar width");
            Design.DisplayNameAttribute.SetDisplayName(properties, "DownColor", "\r\r\rColor for down bars");
            Design.DisplayNameAttribute.SetDisplayName(properties, "Pen", "\r\r\rCandle outline");
            Design.DisplayNameAttribute.SetDisplayName(properties, "Pen2", "\r\r\rWick");
            Design.DisplayNameAttribute.SetDisplayName(properties, "UpColor", "\r\r\rColor for up bars");

            return properties;
        }

        public override bool IsTransparent
        {
            get { return UpColor == Color.Transparent && DownColor == Color.Transparent && Pen.Color == Color.Transparent; }
        }

        public override void PaintBars(ChartControl chartControl, Graphics graphics, Data.Bars bars, int panelIdx, int fromIdx, int toIdx, Rectangle bounds, double max, double min)
        {
            _downBrush.Color = DownColor;

            _upBrush.Color = UpColor;

            int barWidthValue = bars.BarsData.ChartStyle.BarWidthUI;
            SolidBrush brush;
            Color oldPenColor = Pen.Color;
            Color pen2Color = Pen2.Color;

            for (int idx = fromIdx; idx <= toIdx; idx++)
            {
                Color barColor = chartControl.GetBarColor(bars, idx);
                int barWidth = GetBarPaintWidth(barWidthValue);
                double closeValue = bars.GetClose(idx);
                int close = chartControl.GetYByValue(bars, closeValue);
                int high = chartControl.GetYByValue(bars, bars.GetHigh(idx));
                int low = chartControl.GetYByValue(bars, bars.GetLow(idx));
                double openValue = bars.GetOpen(idx);
                double lastCloseValue = idx > 0 ? bars.GetClose(idx - 1) : openValue;
                int lastDirection = idx == 0 || bars.Period.Value2 == 1 && bars.IsFirstBarOfSession(idx) ? 0 : Math.Sign(lastCloseValue - bars.GetOpen(idx - 1));

                switch (lastDirection)
                {
                    case 1:
                        openValue = closeValue > lastCloseValue ? lastCloseValue : closeValue < lastCloseValue - bars.Period.Value * bars.GetTickSize(idx) ? lastCloseValue - bars.Period.Value * bars.GetTickSize(idx) : closeValue;
                        break;
                    case -1:
                        openValue = closeValue > lastCloseValue + bars.Period.Value * bars.GetTickSize(idx) ? lastCloseValue + bars.Period.Value * bars.GetTickSize(idx) : closeValue < lastCloseValue ? lastCloseValue  : closeValue;
                        break;
                    default:
                        break;
                }

                int open = chartControl.GetYByValue(bars, openValue);
                int x = chartControl.GetXByBarIdx(bars, idx);

                Color candleOutlineColorScript = chartControl.GetCandleOutlineColor(bars, idx);
                bool isCandleOutlineColorScriptSet = (chartControl.GetCandleOutlineColor(bars, idx) != Color.Empty);

                Pen.Color = oldPenColor;
                if (barColor != Color.Empty)
                    Pen.Color = (candleOutlineColorScript != Color.Empty ? candleOutlineColorScript : barColor);
                else if (candleOutlineColorScript != Color.Empty)
                    Pen.Color = candleOutlineColorScript;

                if (open == close)
                    graphics.DrawLine(Pen, x - barWidth / 2, close, x + barWidth / 2, close);
                else
                {
                    brush = closeValue >= openValue ? _upBrush : _downBrush;
                    Color oldColor = brush.Color;
                    if (barColor != Color.Empty)
                        brush.Color = barColor;

                    graphics.FillRectangle(brush, x - barWidth / 2 + 1, Math.Min(close, open) + 1,
                        barWidth - 1, Math.Max(open, close) - Math.Min(close, open) - 1);

                    if (barColor != Color.Empty)
                        brush.Color = oldColor;

                    graphics.DrawRectangle(Pen, x - (barWidth / 2) + (Pen.Width / 2), Math.Min(close, open), barWidth - Pen.Width, Math.Max(open, close) - Math.Min(close, open));
                }

                Pen2.Color = isCandleOutlineColorScriptSet ? candleOutlineColorScript : pen2Color;

                if (high < Math.Min(open, close))
                    graphics.DrawLine(Pen2, x, high, x, Math.Min(open, close));

                if (low > Math.Max(open, close))
                    graphics.DrawLine(Pen2, x, low, x, Math.Max(open, close));
            }
            Pen.Color = oldPenColor;
            Pen2.Color = pen2Color;
        }
    }
}
